
#line 3 "<stdout>"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer pycodeYY_create_buffer
#define yy_delete_buffer pycodeYY_delete_buffer
#define yy_flex_debug pycodeYY_flex_debug
#define yy_init_buffer pycodeYY_init_buffer
#define yy_flush_buffer pycodeYY_flush_buffer
#define yy_load_buffer_state pycodeYY_load_buffer_state
#define yy_switch_to_buffer pycodeYY_switch_to_buffer
#define yyin pycodeYYin
#define yyleng pycodeYYleng
#define yylex pycodeYYlex
#define yylineno pycodeYYlineno
#define yyout pycodeYYout
#define yyrestart pycodeYYrestart
#define yytext pycodeYYtext
#define yywrap pycodeYYwrap
#define yyalloc pycodeYYalloc
#define yyrealloc pycodeYYrealloc
#define yyfree pycodeYYfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;
#endif /* ! C99 */

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE pycodeYYrestart(pycodeYYin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 262144
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

extern yy_size_t pycodeYYleng;

extern FILE *pycodeYYin, *pycodeYYout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up pycodeYYtext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up pycodeYYtext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	yy_size_t yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via pycodeYYrestart()), so that the user can continue scanning by
	 * just pointing pycodeYYin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when pycodeYYtext is formed. */
static char yy_hold_char;
static yy_size_t yy_n_chars;		/* number of characters read into yy_ch_buf */
yy_size_t pycodeYYleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow pycodeYYwrap()'s to do buffer switches
 * instead of setting up a fresh pycodeYYin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void pycodeYYrestart (FILE *input_file  );
void pycodeYY_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE pycodeYY_create_buffer (FILE *file,int size  );
void pycodeYY_delete_buffer (YY_BUFFER_STATE b  );
void pycodeYY_flush_buffer (YY_BUFFER_STATE b  );
void pycodeYYpush_buffer_state (YY_BUFFER_STATE new_buffer  );
void pycodeYYpop_buffer_state (void );

static void pycodeYYensure_buffer_stack (void );
static void pycodeYY_load_buffer_state (void );
static void pycodeYY_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER pycodeYY_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE pycodeYY_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE pycodeYY_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE pycodeYY_scan_bytes (yyconst char *bytes,yy_size_t len  );

void *pycodeYYalloc (yy_size_t  );
void *pycodeYYrealloc (void *,yy_size_t  );
void pycodeYYfree (void *  );

#define yy_new_buffer pycodeYY_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        pycodeYYensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            pycodeYY_create_buffer(pycodeYYin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        pycodeYYensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            pycodeYY_create_buffer(pycodeYYin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

#define pycodeYYwrap(n) 1
#define YY_SKIP_YYWRAP

typedef unsigned char YY_CHAR;

FILE *pycodeYYin = (FILE *) 0, *pycodeYYout = (FILE *) 0;

typedef int yy_state_type;

extern int pycodeYYlineno;

int pycodeYYlineno = 1;

extern char *pycodeYYtext;
#define yytext_ptr pycodeYYtext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up pycodeYYtext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	pycodeYYleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 59
#define YY_END_OF_BUFFER 60
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_acclist[949] =
    {   0,
    16414,16414,   60,   58,   59,   57,   58,   59,   56,   59,
       54,   58,   59,   55,   58,   59,   53,   58,   59,   58,
       59,   58,   59,   58,   59,   33,   56,   59,   19,   32,
       58,   59,   19,   32,   58,   59,   19,   32,   58,   59,
       19,   32,   58,   59,   19,   32,   58,   59,   19,   32,
       58,   59,   19,   32,   58,   59,   19,   32,   58,   59,
       19,   32,   58,   59,   19,   32,   58,   59,   19,   32,
       58,   59,   19,   32,   58,   59,   19,   32,   58,   59,
       19,   32,   58,   59,   19,   32,   58,   59,   19,   32,
       58,   59,   19,   32,   58,   59,   19,   32,   58,   59,

       19,   32,   58,   59,   19,   32,   58,   59,   19,   32,
       58,   59,   19,   32,   58,   59,   19,   32,   58,   59,
       19,   32,   58,   59,   57,   58,   59,   11,   58,   59,
       10,   58,   59,   10,   58,   59,   10,   58,   59,   10,
       58,   59,   13,   58,   59,   12,   57,   58,   59,   13,
       54,   58,   59,   13,   55,   58,   59,   13,   53,   58,
       59,   14,   58,   59,   12,   13,   58,   59,   15,   58,
       59,   13,   58,   59,   13,   58,   59,   13,   58,   59,
       13,   58,   59,    6,   58,   59,    6,   58,   59,    6,
       58,   59,    6,   58,   59,    7,   57,   58,   59,    7,

       58,   59,    9,   58,   59,    8,   58,   59,    8,   58,
       59,    8,   58,   59,    8,   58,   59,   31,   56,   59,
       19,   58,   59,   19,   58,   59,   19,   58,   59,   19,
       58,   59,   19,   58,   59,   19,   58,   59,   19,   58,
       59,   19,   58,   59,   19,   58,   59,   19,   58,   59,
       19,   58,   59,   19,   58,   59,   19,   58,   59,   19,
       58,   59,   19,   58,   59,   19,   58,   59,   19,   58,
       59,   19,   58,   59,   19,   58,   59,   19,   58,   59,
       19,   58,   59,   19,   58,   59,   19,   58,   59,   19,
       58,   59,   27,   57,   58,   59,   26,   56,   59,   20,

       57,   58,   59,   25,   56,   59,   24,   55,   58,   59,
       23,   58,   59,   23,   58,   59,   23,   58,   59,   23,
       58,   59,   23,   58,   59,   23,   58,   59,   23,   58,
       59,   23,   58,   59,   23,   58,   59,   23,   58,   59,
       23,   58,   59,   23,   58,   59,   23,   58,   59,   23,
       58,   59,   23,   58,   59,   23,   58,   59,   23,   58,
       59,   23,   58,   59,   23,   58,   59,   23,   58,   59,
       23,   58,   59,   23,   58,   59,   57,   58,   59,16412,
       29,   56,   59, 8222,   58,   59, 8222,   55,   58,   59,
     8222,   58,   59, 8222,   58,   59, 8222,   58,   59,   38,

       39,   58,   59,   38,   39,   57,   58,   59,   39,   54,
       58,   59,   38,   39,   55,   58,   59,   37,   39,   53,
       58,   59,   38,   39,   58,   59,   38,   39,   58,   59,
       39,   58,   59,   38,   39,   58,   59,   44,   45,   58,
       59,   44,   45,   57,   58,   59,   43,   45,   54,   58,
       59,   44,   45,   55,   58,   59,   45,   53,   58,   59,
       44,   45,   58,   59,   44,   45,   58,   59,   45,   58,
       59,   44,   45,   58,   59,   48,   50,   58,   59,   48,
       50,   57,   58,   59,   48,   49,   56,   59,   48,   50,
       54,   58,   59,   48,   50,   55,   58,   59,   48,   50,

       53,   58,   59,   48,   50,   58,   59,   48,   50,   58,
       59,   50,   58,   59,   48,   50,   58,   59,   57,   55,
       54,   53,   18,   19,   32,   19,   32,   19,   32,   19,
       32,   19,   32,   19,   32,   16,   19,   32,   19,   32,
       19,   32,   19,   32,   19,   32,   19,   32,   19,   32,
       19,   32,   19,   32,   19,   32,   19,   32,   17,   19,
       32,   19,   32,   17,   19,   32,   19,   32,   19,   32,
       19,   32,   19,   32,   19,   32,   19,   32,   19,   32,
       19,   32,   19,   32,   19,   32,   57,   11,   10,   10,
       13,   12,   57,   13,   13,   55,   13,   13,   13,   54,

       13,   53,   13,    6,    6,    7,   57,    8,    8,   19,
       19,   19,   19,   19,   19,   16,   19,   19,   19,   19,
       19,   19,   19,   19,   19,   19,   19,   17,   19,   19,
       17,   19,   19,   19,   19,   19,   19,   19,   19,   19,
       19,   19,   27,   57,   26,   20,   57,   24,   55,   23,
       23,   23,   23,   23,   21,   23,   23,   23,   23,   23,
       23,   23,   23,   23,   23,   23,   22,   23,   23,   22,
       23,   23,   23,   23,   23,   23,   23,   23,   23,   23,
       57,16412,   29, 8220,   38,   38,   57,   38,   55,   54,
       38,   35,   35,   34,   44,   44,   57,   44,   55,   53,

       44,   41,   41,   40,   48,   48,   57,   48,   48,   55,
       55,   48,   48,   52,   51,   19,   32,   19,   32,   19,
       32,   19,   32,   19,   32,   19,   32,   19,   32,   16,
       19,   32,   19,   32,   19,   32,   19,   32,   19,   32,
       19,   32,   19,   32,   19,   32,   19,   32,   19,   32,
       19,   32,   19,   32,   19,   32,   19,   32,   19,   32,
       19,   32,   13,   52,   13,   51,   19,   19,   19,   19,
       19,   19,   19,   16,   19,   19,   19,   19,   19,   19,
       19,   19,   19,   19,   19,   19,   19,   19,   19,   19,
       23,   23,   23,   23,   23,   23,   21,   23,   23,   23,

       23,   23,   23,   23,   23,   23,   23,   23,   23,   23,
       23,   23,   36,   52,   42,   51,   46,   52,   47,   51,
       19,   32,    3,   19,   32,   16,   19,   32,   19,   32,
       19,   32,   19,   32,   19,   32,    1,   19,   32,   19,
       32,   19,   32,   19,   32,   19,   32,   19,   32,   19,
       32,   19,   32,   19,   32,   19,   32,   19,   32,   19,
        3,   19,   16,   19,   19,   19,   19,   19,   19,   19,
       19,   19,   19,   19,   19,   19,   19,   19,   19,   23,
       23,   23,   23,   23,   23,   23,   23,   23,   23,   23,
       23,   23,   23,   23,   19,   32,   16,   19,   32,   19,

       32,   19,   32,   19,   32,   19,   32,   19,   32,   19,
       32,   19,   32,   19,   16,   19,   19,   19,   19,   19,
       19,   19,   19,   23,   23,   23,   23,   23,   23,   23,
       23,    2,   19,   32,   19,   32,    5,   19,   19,   23,
       23,   19,   32,    4,   18,    5,   19,   23
    } ;

static yyconst flex_int16_t yy_accept[503] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    2,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    4,    6,    9,   11,   14,   17,   20,   22,   24,
       26,   29,   33,   37,   41,   45,   49,   53,   57,   61,
       65,   69,   73,   77,   81,   85,   89,   93,   97,  101,
      105,  109,  113,  117,  121,  125,  128,  131,  134,  137,
      140,  143,  146,  150,  154,  158,  162,  165,  169,  172,
      175,  178,  181,  184,  187,  190,  193,  196,  200,  203,
      206,  209,  212,  215,  218,  221,  224,  227,  230,  233,

      236,  239,  242,  245,  248,  251,  254,  257,  260,  263,
      266,  269,  272,  275,  278,  281,  284,  287,  290,  293,
      297,  300,  304,  307,  311,  314,  317,  320,  323,  326,
      329,  332,  335,  338,  341,  344,  347,  350,  353,  356,
      359,  362,  365,  368,  371,  374,  377,  381,  384,  387,
      391,  394,  397,  400,  404,  409,  413,  418,  423,  427,
      431,  434,  438,  442,  447,  452,  457,  461,  465,  469,
      472,  476,  480,  485,  489,  494,  499,  504,  508,  512,
      515,  519,  520,  520,  521,  521,  522,  523,  523,  524,
      524,  526,  528,  530,  532,  534,  536,  539,  541,  543,

      545,  547,  549,  551,  553,  555,  557,  559,  562,  564,
      567,  569,  571,  573,  575,  577,  579,  581,  583,  585,
      587,  588,  589,  590,  591,  592,  594,  595,  597,  598,
      599,  601,  603,  604,  605,  606,  608,  609,  610,  611,
      612,  613,  614,  615,  616,  618,  619,  620,  621,  622,
      623,  624,  625,  626,  627,  628,  630,  631,  633,  634,
      635,  636,  637,  638,  639,  640,  641,  642,  643,  645,
      646,  648,  650,  651,  652,  653,  654,  655,  657,  658,
      659,  660,  661,  662,  663,  664,  665,  666,  667,  669,
      670,  672,  673,  674,  675,  676,  677,  678,  679,  680,

      681,  683,  684,  685,  686,  688,  688,  690,  691,  692,
      693,  694,  695,  696,  698,  700,  700,  701,  702,  703,
      704,  705,  706,  706,  708,  709,  711,  712,  713,  714,
      715,  716,  716,  718,  720,  722,  724,  726,  728,  730,
      733,  735,  737,  739,  741,  743,  745,  747,  749,  751,
      753,  755,  757,  759,  761,  763,  765,  767,  768,  769,
      770,  771,  772,  773,  774,  776,  777,  778,  779,  780,
      781,  782,  783,  784,  785,  786,  787,  788,  789,  790,
      791,  792,  793,  794,  795,  796,  797,  799,  800,  801,
      802,  803,  804,  805,  806,  807,  808,  809,  810,  811,

      812,  813,  815,  815,  817,  817,  819,  821,  821,  823,
      826,  829,  831,  833,  835,  837,  838,  840,  842,  844,
      846,  848,  850,  852,  854,  856,  858,  860,  861,  863,
      865,  866,  867,  868,  869,  870,  871,  872,  873,  874,
      875,  876,  877,  878,  879,  880,  881,  882,  883,  884,
      885,  886,  887,  888,  889,  890,  891,  892,  893,  894,
      895,  897,  900,  902,  904,  906,  908,  910,  912,  914,
      914,  915,  917,  918,  919,  920,  921,  922,  923,  924,
      925,  926,  927,  928,  929,  930,  931,  932,  933,  935,
      937,  938,  939,  940,  941,  942,  944,  946,  947,  948,

      949,  949
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    4,    5,    6,    1,    4,    4,    7,    8,
        9,    4,    4,   10,    4,   11,    4,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,   13,    4,    4,
        4,    4,    1,    1,   14,   14,   14,   14,   14,   15,
       14,   14,   14,   14,   14,   14,   14,   16,   14,   14,
       14,   17,   14,   18,   19,   14,   14,   14,   14,   14,
        4,   20,    4,    1,   21,    4,   22,   23,   24,   25,

       26,   27,   28,   29,   30,   14,   31,   32,   33,   34,
       35,   36,   14,   37,   38,   39,   40,   14,   41,   42,
       43,   14,    4,    4,    4,    4,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[44] =
    {   0,
        1,    2,    3,    4,    5,    4,    5,    6,    2,    4,
        7,    8,    2,    9,    9,    9,    9,    9,    9,   10,
        9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
        9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
        9,    9,    9
    } ;

static yyconst flex_int16_t yy_base[531] =
    {   0,
        0,   39,   79,    0,  122,    0,  165,    0,  208,    0,
      251,    0,  294,    0,    6,    8,  337,    0,  380,  420,
        0,    0,    0,    0,  459,  498,  537,  576,  615,  654,
     1541, 1623, 1537, 1623, 1526,    0, 1511,    7,   11,   15,
     1623,   13,   25,   15,   46,   23,  387,   27,  382,  380,
       51,  398,  399,  400,  420,   56,  433,  405,  437,  462,
      392,  435,  475,  427,   19,   61, 1623,    0,    8,  470,
      504,    0, 1501, 1495,  489, 1492, 1623,    0, 1623,  694,
      714,   38,  434,    0,   61,  509,  515, 1488, 1623, 1623,
        0,   65,  543,  548, 1623,   63,  441,  502,  520,  482,

      592,  525,  550,  556,  558,  560,  593,  562,  604,  564,
      616,  599,  603,  636,  637,  638,  659,  597,  642,  399,
     1623, 1481, 1623,    0,    0, 1458,  501, 1442,  672,    5,
     1427,  633, 1384,  453,  648, 1377,  496, 1386, 1372, 1369,
      525,  681, 1368,  675, 1374, 1372,  587, 1623, 1623,    0,
      544,  706,  721,    0, 1398, 1394,  722, 1391,  701,  728,
      714,  732,    0, 1395, 1389,  739, 1386,  725,  743,  738,
      746, 1372,  394, 1371, 1384,  759, 1381,  747,  765,  768,
      769, 1385, 1379,    0, 1367, 1363, 1359,  750, 1623,    0,
      673,  773,  779,  682,  786,  784,  788,  790,  792,  796,

      800,  804,  809,  810,  811,  817,  821,  827,  828,  829,
      835,  839,  843,  847,  849,  851,  855,  859,  861,  863,
      631, 1623,    0,  868,    0, 1362, 1357,  884, 1345,  875,
     1342, 1332,    0,    0,  878, 1327,    0,  889,  887,  892,
      893,  897,  902,  903,  904,  908,  909,  910,  914,  915,
      922,  921,  927,  928,  932,  939,  940,  941,  946,  948,
      950,  954,  960,  958,  962,  964,  966,  972,  775, 1623,
     1312,    0,    0, 1274, 1254,  984, 1259, 1242, 1253, 1252,
     1235, 1216,  451, 1214, 1202, 1193, 1188, 1187,    0, 1180,
        0, 1160, 1153, 1153, 1155, 1148, 1131, 1125, 1127, 1110,

      821, 1623, 1623,    0, 1129, 1122,  988, 1118,  994, 1623,
      979, 1623,    0, 1114,  995, 1102, 1098,  998, 1623,  983,
     1623, 1083, 1006,  667, 1094, 1007, 1084, 1074,  999, 1623,
     1623, 1009, 1010, 1014, 1015, 1020, 1021, 1022, 1026, 1034,
     1027, 1036, 1041, 1042, 1047, 1048, 1055, 1061, 1062, 1067,
     1066, 1068, 1075, 1074, 1080,    0,    0, 1081, 1085, 1087,
     1099, 1106, 1086, 1107, 1118, 1122, 1124, 1126, 1131, 1130,
     1132, 1136, 1137, 1143, 1148, 1150, 1154, 1156, 1158, 1164,
     1030, 1035, 1031, 1029, 1001,  985,    0,  989,  966,  952,
      938,  873,  876,  853,  861,  838,  831,  823,  813,  817,

      813, 1623, 1119, 1623, 1171, 1623, 1623, 1168, 1169, 1176,
     1178, 1189, 1190, 1191, 1195,  839, 1196, 1201, 1197, 1202,
     1203, 1207, 1209, 1216, 1223, 1229, 1233, 1234, 1239, 1240,
     1241, 1246, 1248, 1253, 1254, 1255, 1259, 1260, 1264, 1265,
     1274, 1280, 1284, 1285, 1290,  805,  773,  757,  730,  731,
      720,  715,  714,  694,  685,  663,  671,  617,  593,  581,
     1291, 1301, 1297, 1299, 1305, 1310, 1311, 1312, 1317,    0,
     1316, 1324, 1325, 1332, 1333, 1337, 1338, 1345, 1346,  559,
      542,  506,  492,  462,  416,  428,  399,  403, 1350, 1352,
     1362, 1364, 1368,  349,  341, 1370, 1623, 1374, 1375,   47,

     1623, 1411, 1421, 1431, 1441, 1451, 1461, 1466, 1468, 1477,
     1487, 1497, 1500, 1502, 1506, 1515, 1518, 1526, 1536, 1545,
     1555, 1565, 1574, 1584, 1594, 1604,   20, 1609,   16, 1613
    } ;

static yyconst flex_int16_t yy_def[531] =
    {   0,
      502,  502,  501,    3,  501,    5,  501,    7,  501,    9,
      501,   11,  501,   13,    2,    2,  501,   17,  503,  503,
        2,    2,    2,    2,  504,  504,  505,  505,  506,  506,
      501,  501,  501,  501,  501,  507,  501,  501,  501,  501,
      501,  508,  508,  508,  508,  508,  508,  508,  508,  508,
      508,  508,  508,  508,  508,  508,  508,  508,  508,  508,
      508,  508,  508,  508,  508,  501,  501,  509,  509,  509,
      509,  510,  501,  510,  511,  510,  501,  510,  501,  512,
      512,   81,   81,  513,  513,  513,  513,  501,  501,  501,
      514,  514,  514,  514,  501,  515,  515,  515,  515,  515,

      515,  515,  515,  515,  515,  515,  515,  515,  515,  515,
      515,  515,  515,  515,  515,  515,  515,  515,  515,  501,
      501,  501,  501,  516,  517,  517,  517,  517,  517,  517,
      517,  517,  517,  517,  517,  517,  517,  517,  517,  517,
      517,  517,  517,  517,  517,  517,  518,  501,  501,  507,
      501,  501,  501,  519,  519,  501,  520,  501,  519,  519,
      521,  519,  522,  522,  501,  523,  501,  522,  522,  524,
      522,  525,  525,  525,  501,  526,  501,  525,  525,  525,
      525,  501,  501,  507,  501,  501,  501,  501,  501,  527,
      508,  508,  508,  508,  508,  508,  508,  508,  508,  508,

      508,  508,  508,  508,  508,  508,  508,  508,  508,  508,
      508,  508,  508,  508,  508,  508,  508,  508,  508,  508,
      501,  501,  509,  509,  510,  501,  510,  511,  510,   81,
      510,  510,   81,  513,  513,  501,  514,  514,  515,  515,
      515,  515,  515,  515,  515,  515,  515,  515,  515,  515,
      515,  515,  515,  515,  515,  515,  515,  515,  515,  515,
      515,  515,  515,  515,  515,  515,  515,  515,  501,  501,
      501,  516,  517,  517,  517,  517,  517,  517,  517,  517,
      517,  517,  517,  517,  517,  517,  517,  517,  517,  517,
      517,  517,  517,  517,  517,  517,  517,  517,  517,  517,

      518,  501,  501,  519,  519,  501,  520,  501,  519,  501,
      501,  501,  522,  522,  523,  501,  501,  522,  501,  501,
      501,  525,  525,  525,  501,  526,  526,  501,  525,  501,
      501,  528,  508,  508,  508,  508,  508,  508,  508,  508,
      508,  508,  508,  508,  508,  508,  508,  508,  508,  508,
      508,  508,  508,  508,  508,  510,  510,  515,  515,  515,
      515,  515,  515,  515,  515,  515,  515,  515,  515,  515,
      515,  515,  515,  515,  515,  515,  515,  515,  515,  515,
      517,  517,  517,  517,  517,  517,  517,  517,  517,  517,
      517,  517,  517,  517,  517,  517,  517,  517,  517,  517,

      517,  501,  501,  501,  501,  501,  501,  528,  508,  508,
      508,  508,  508,  508,  508,  501,  508,  508,  508,  508,
      508,  508,  508,  508,  508,  508,  508,  515,  515,  515,
      515,  515,  515,  515,  515,  515,  515,  515,  515,  515,
      515,  515,  515,  515,  515,  517,  517,  517,  517,  517,
      517,  517,  517,  517,  517,  517,  517,  517,  517,  517,
      508,  508,  508,  508,  508,  508,  508,  508,  508,  529,
      515,  515,  515,  515,  515,  515,  515,  515,  515,  517,
      517,  517,  517,  517,  517,  517,  517,  501,  508,  508,
      530,  515,  515,  517,  517,  508,  501,  530,  515,  517,

        0,  501,  501,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501,  501,  501,  501,  501
    } ;

static yyconst flex_int16_t yy_nxt[1667] =
    {   0,
      501,   33,   34,  501,   35,   36,   37,  120,  121,  120,
      121,  186,  186,  187,  187,  186,   38,  187,   39,  186,
      189,  187,  189,  190,  491,  190,  189,  188,  332,  190,
      189,  188,  189,  190,  189,  190,   38,  190,  277,   40,
       33,   34,  278,   35,   36,   37,  192,  188,  220,  193,
      186,  188,  187,  189,  233,   38,  190,   39,  189,  194,
      196,  190,  221,  189,  197,  186,  190,  187,  222,  186,
      189,  187,  291,  190,  233,   38,  201,  211,   40,   32,
       33,   41,   32,   35,   36,   37,   32,   32,   32,   32,
       32,   32,   42,   43,   44,   45,   46,   47,   32,   42,

       48,   49,   50,   51,   52,   53,   54,   42,   55,   42,
       56,   42,   57,   58,   59,   60,   61,   62,   63,   64,
       42,   65,   32,   66,   34,   32,   35,   36,   37,   67,
       32,   32,   32,   32,   32,   68,   68,   68,   69,   68,
       70,   32,   68,   68,   68,   68,   68,   68,   68,   68,
       68,   68,   68,   68,   68,   68,   68,   68,   69,   68,
       68,   71,   68,   68,   68,   72,   73,   34,   72,   74,
       75,   76,   32,   77,   78,   72,   72,   79,   80,   80,
       80,   81,   80,   82,   72,   80,   80,   80,   80,   80,
       80,   80,   80,   80,   80,   80,   80,   80,   80,   80,

       80,   81,   80,   80,   83,   80,   80,   80,   32,   33,
       34,   32,   35,   36,   37,   32,   32,   32,   32,   32,
       32,   84,   84,   84,   85,   84,   86,   32,   84,   84,
       84,   84,   84,   84,   84,   84,   84,   84,   84,   84,
       84,   84,   84,   84,   85,   84,   84,   87,   84,   84,
       84,   32,   88,   34,   32,   35,   36,   37,   89,   89,
       89,   32,   32,   90,   91,   91,   91,   92,   91,   93,
       32,   91,   91,   91,   91,   91,   91,   91,   91,   91,
       91,   91,   91,   91,   91,   91,   91,   92,   91,   91,
       94,   91,   91,   91,   32,   33,   95,   32,   35,   36,

       37,   32,   32,   32,   32,   32,   32,   96,   97,   98,
       99,  100,  101,   32,   96,  102,  103,  104,  105,  106,
      107,  108,   96,  109,   96,  110,   96,  111,  112,  113,
      114,  115,  116,  117,  118,   96,  119,   32,  122,  123,
       32,   35,  124,   37,   32,   32,   32,   32,   32,   32,
      125,  126,  125,  127,  128,  129,   32,  125,  130,  131,
      132,  133,  134,  135,  136,  125,  137,  125,  138,  125,
      139,  140,  141,  142,  125,  143,  144,  145,  125,  146,
       32,  147,  148,  291,   35,  150,   37,  189,  500,  189,
      190,  186,  190,  187,  189,  324,  151,  190,  152,  189,

      269,  270,  190,  195,  488,  189,  189,  189,  190,  190,
      190,  199,  189,  323,  200,  190,  151,  217,  198,  153,
       32,  147,  148,  195,   35,  150,   37,  189,  204,  202,
      190,  207,  291,  205,  189,  206,  151,  190,  152,  203,
      189,  210,  189,  190,  189,  190,  208,  190,  189,  387,
      233,  190,  209,  210,  387,  219,  151,  210,  213,  153,
      155,   34,  240,  156,  157,  158,  186,  212,  187,  189,
      233,  218,  190,  214,  186,  159,  187,  160,  161,  186,
      388,  187,  189,  215,  283,  190,  224,  216,  389,  189,
      184,  195,  190,  387,  284,  159,  184,  184,  162,  155,

       34,  184,  156,  157,  158,  186,  224,  187,  186,  189,
      187,  195,  190,  186,  159,  187,  160,  161,  242,  186,
      224,  187,  289,  495,  186,  235,  187,  189,  290,  291,
      190,  235,  189,  291,  159,  190,  241,  162,  164,   34,
      224,  165,  166,  167,  291,  235,  294,  186,  186,  187,
      187,  235,  186,  168,  187,  169,  170,  189,  244,  238,
      190,  295,  245,  189,  238,  189,  190,  189,  190,  189,
      190,  189,  190,  168,  190,  494,  171,  164,   34,  238,
      165,  166,  167,  249,  238,  259,  246,  247,  301,  302,
      248,  250,  168,  255,  169,  170,  186,  387,  187,  189,

      189,  251,  190,  190,  189,  291,  189,  190,  243,  190,
      189,  189,  168,  190,  190,  171,  173,  174,  291,  175,
      176,  177,  252,  189,  261,  267,  190,  253,  243,  254,
      256,  178,  221,  179,  180,  258,  257,  258,  222,  262,
      186,  258,  187,  189,  189,  189,  190,  190,  190,  189,
      260,  178,  190,  487,  181,  173,  174,  263,  175,  176,
      177,  264,  265,  186,  280,  187,  189,  281,  324,  190,
      178,  268,  179,  180,  266,  243,  186,  285,  187,  186,
      189,  187,  286,  190,  287,  186,  323,  187,  276,  189,
      178,  276,  190,  181,  225,  243,  291,  225,  225,  225,

      225,  291,  296,  225,  225,  308,  297,  187,  276,  486,
      186,  276,  187,  225,  225,  311,  312,  225,  231,  225,
      232,  335,  188,  225,  225,  186,  184,  187,  184,  186,
      485,  317,  308,  225,  187,  484,  308,  188,  187,  320,
      321,  184,  188,  184,  309,  184,  483,  186,  309,  317,
      186,  186,  317,  187,  186,  482,  187,  188,  184,  318,
      481,  322,  318,  184,  309,  184,  323,  387,  309,  186,
      501,  187,  322,  186,  322,  187,  269,  270,  327,  318,
      189,  329,  318,  190,  323,  329,  189,  291,  323,  190,
      186,  189,  187,  189,  190,  189,  190,  189,  190,  189,

      190,  329,  190,  189,  333,  329,  190,  189,  210,  480,
      190,  189,  334,  338,  190,  337,  189,  189,  189,  190,
      190,  190,  301,  302,  189,  336,  340,  190,  189,  339,
      387,  190,  343,  341,  189,  189,  189,  190,  190,  190,
      416,  342,  189,  344,  460,  190,  189,  210,  459,  190,
      189,  345,  458,  190,  189,  346,  189,  190,  189,  190,
      457,  190,  189,  347,  456,  190,  189,  348,  189,  190,
      189,  190,  186,  190,  187,  291,  350,  210,  351,  225,
      349,  225,  186,  455,  187,  184,  353,  454,  355,  352,
      354,  184,  184,  186,  189,  187,  184,  190,  453,  189,

      189,  210,  190,  190,  189,  387,  186,  190,  187,  189,
      189,  189,  190,  190,  190,  189,  189,  189,  190,  190,
      190,  189,  189,  358,  190,  190,  359,  258,  189,  189,
      363,  190,  190,  362,  189,  189,  360,  190,  190,  189,
      365,  361,  190,  364,  366,  368,  189,  189,  189,  190,
      190,  190,  367,  189,  369,  189,  190,  189,  190,  452,
      190,  189,  370,  258,  190,  189,  371,  189,  190,  189,
      190,  189,  190,  189,  190,  372,  190,  451,  373,  189,
      403,  312,  190,  375,  405,  321,  258,  374,  186,  376,
      187,  291,  184,  378,  184,  379,  377,  380,  308,  184,

      187,  184,  186,  186,  317,  187,  258,  184,  501,  322,
      322,  184,  322,  184,  184,  291,  189,  189,  323,  190,
      190,  189,  189,  450,  190,  190,  327,  189,  189,  189,
      190,  190,  190,  189,  189,  416,  190,  190,  449,  410,
      411,  189,  413,  189,  190,  412,  190,  409,  189,  189,
      448,  190,  190,  210,  189,  189,  447,  190,  190,  414,
      387,  210,  189,  418,  415,  190,  417,  446,  189,  189,
      419,  190,  190,  189,  189,  189,  190,  190,  190,  411,
      407,  189,  189,  421,  190,  190,  501,  189,  189,  420,
      190,  190,  189,  189,  189,  190,  190,  190,  406,  210,

      422,  425,  323,  423,  316,  426,  189,  424,  404,  190,
      429,  427,  430,  189,  189,  314,  190,  190,  428,  416,
      403,  312,  306,  433,  431,  189,  402,  432,  190,  189,
      305,  189,  190,  189,  190,  401,  190,  189,  189,  189,
      190,  190,  190,  189,  189,  434,  190,  190,  258,  258,
      189,  435,  436,  190,  437,  189,  400,  189,  190,  439,
      190,  189,  430,  189,  190,  189,  190,  291,  190,  399,
      438,  189,  405,  321,  190,  189,  189,  398,  190,  190,
      258,  440,  443,  189,  397,  189,  190,  441,  190,  444,
      396,  291,  395,  442,  411,  445,  189,  189,  189,  190,

      190,  190,  189,  189,  189,  190,  190,  190,  189,  189,
      189,  190,  190,  190,  189,  394,  189,  190,  466,  190,
      210,  393,  392,  189,  463,  461,  190,  468,  462,  291,
      189,  464,  465,  470,  210,  391,  189,  390,  467,  190,
      189,  189,  387,  190,  190,  210,  189,  189,  189,  190,
      190,  190,  469,  189,  210,  189,  190,  210,  190,  430,
      189,  189,  189,  190,  190,  190,  189,  189,  386,  190,
      190,  189,  189,  385,  190,  190,  258,  471,  384,  383,
      476,  189,  473,  291,  190,  472,  475,  189,  478,  474,
      190,  189,  189,  382,  470,  190,  477,  189,  189,  258,

      190,  190,  488,  258,  189,  381,  189,  190,  189,  190,
      258,  190,  189,  271,  258,  190,  479,  189,  189,  189,
      190,  190,  190,  189,  189,  488,  190,  190,  236,  411,
      489,  189,  189,  411,  190,  190,  490,  210,  229,  189,
      189,  411,  190,  190,  189,  189,  227,  190,  190,  411,
      210,  357,  189,  189,  430,  190,  190,  189,  492,  189,
      190,  356,  190,  226,  493,  185,  430,  183,  430,  497,
      258,  189,  190,  331,  190,  189,  430,  189,  190,  258,
      190,  497,  189,  330,  190,  190,  182,  328,  325,  496,
      323,  323,  316,  183,  210,  210,  314,  185,  306,  305,

      258,  300,  299,  499,  298,  291,  293,  292,  288,  282,
      258,   32,   32,   32,   32,   32,   32,   32,   32,   32,
       32,  149,  149,  149,  149,  149,  149,  149,  149,  149,
      149,  154,  154,  154,  154,  154,  154,  154,  154,  154,
      154,  163,  163,  163,  163,  163,  163,  163,  163,  163,
      163,  172,  172,  172,  172,  172,  172,  172,  172,  172,
      172,  184,  184,  279,  184,  184,  184,  184,  184,  184,
      184,  191,  191,  191,  191,  223,  223,  225,  275,  274,
      225,  225,  271,  225,  225,  225,  225,  228,  228,  236,
      228,  228,  228,  228,  228,  228,  228,  230,  229,  227,

      230,  230,  226,  230,  230,  230,  230,  234,  234,  237,
      237,  239,  239,  239,  239,  272,  272,  185,  272,  272,
      272,  272,  272,  272,  272,  273,  273,  303,  303,  303,
      183,  303,  303,  303,  303,  303,  304,  304,  182,  304,
      501,  304,  304,  304,  304,  307,  307,  501,  307,  307,
      307,  307,  307,  307,  307,  310,  310,  310,  310,  310,
      310,  310,  310,  310,  310,  313,  313,  501,  313,  501,
      313,  313,  313,  313,  315,  315,  501,  315,  315,  315,
      315,  315,  315,  315,  319,  319,  319,  319,  319,  319,
      319,  319,  319,  319,  322,  322,  322,  322,  501,  322,

      322,  322,  322,  322,  326,  326,  326,  326,  326,  326,
      326,  326,  326,  326,  408,  408,  408,  408,  498,  498,
      498,  498,   31,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501
    } ;

static yyconst flex_int16_t yy_chk[1667] =
    {   0,
        0,    1,    1,    0,    1,    1,    1,   15,   15,   16,
       16,   38,   69,   38,   69,   39,    1,   39,    1,   40,
       42,   40,   44,   42,  529,   44,   65,   39,  527,   65,
       46,   40,   43,   46,   48,   43,    1,   48,  130,    1,
        2,    2,  130,    2,    2,    2,   43,   39,   65,   44,
       45,   40,   45,   45,   82,    2,   45,    2,   51,   46,
       48,   51,   66,   56,   48,   85,   56,   85,   66,   92,
       96,   92,  500,   96,   82,    2,   51,   56,    2,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,

        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,

        7,    7,    7,    7,    7,    7,    7,    7,    9,    9,
        9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
        9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
        9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
        9,    9,    9,    9,    9,    9,    9,    9,    9,    9,
        9,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   13,   13,   13,   13,   13,   13,

       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   17,   17,   17,
       17,   17,   17,   17,   17,   17,   17,   17,   17,   17,
       17,   17,   17,   17,   17,   17,   17,   17,   17,   17,
       17,   17,   17,   17,   17,   17,   17,   17,   17,   17,
       17,   17,   17,   17,   17,   17,   17,   17,   17,   17,
       19,   19,   19,  495,   19,   19,   19,   50,  494,   49,
       50,   47,   49,   47,   47,  173,   19,   47,   19,   61,

      120,  120,   61,   47,  488,   52,   53,   54,   52,   53,
       54,   50,   58,  173,   50,   58,   19,   61,   49,   19,
       20,   20,   20,   47,   20,   20,   20,   55,   53,   52,
       55,   54,  487,   53,   64,   53,   20,   64,   20,   52,
       57,   58,   62,   57,   59,   62,   55,   59,   97,  486,
       83,   97,   55,   55,  485,   64,   20,   55,   59,   20,
       25,   25,   97,   25,   25,   25,   60,   57,   60,   60,
       83,   62,   60,   59,   70,   25,   70,   25,   25,   63,
      283,   63,   63,   60,  134,   63,   70,   60,  283,  100,
       75,   63,  100,  484,  134,   25,   75,   75,   25,   26,

       26,   75,   26,   26,   26,  127,   70,  127,   71,   98,
       71,   63,   98,   86,   26,   86,   26,   26,  100,   87,
       71,   87,  137,  483,   99,   86,   99,   99,  137,  137,
       99,   87,  102,  137,   26,  102,   98,   26,   27,   27,
       71,   27,   27,   27,  482,   86,  141,   93,  151,   93,
      151,   87,   94,   27,   94,   27,   27,  103,  102,   93,
      103,  141,  102,  104,   94,  105,  104,  106,  105,  108,
      106,  110,  108,   27,  110,  481,   27,   28,   28,   93,
       28,   28,   28,  105,   94,  110,  103,  104,  147,  147,
      104,  106,   28,  108,   28,   28,  101,  480,  101,  101,

      107,  106,  101,  107,  118,  460,  112,  118,  101,  112,
      113,  109,   28,  113,  109,   28,   29,   29,  459,   29,
       29,   29,  107,  111,  113,  118,  111,  107,  101,  107,
      109,   29,  221,   29,   29,  112,  109,  109,  221,  113,
      114,  109,  114,  114,  115,  116,  114,  115,  116,  119,
      111,   29,  119,  458,   29,   30,   30,  114,   30,   30,
       30,  114,  115,  117,  132,  117,  117,  132,  324,  117,
       30,  119,   30,   30,  116,  117,  129,  135,  129,  144,
      191,  144,  135,  191,  135,  142,  324,  142,  129,  194,
       30,  144,  194,   30,   80,  117,  457,   80,   80,   80,

       80,  456,  142,   80,   80,  159,  142,  159,  129,  455,
      152,  144,  152,   80,   81,  161,  161,   81,   81,   81,
       81,  194,  152,   81,   81,  153,  157,  153,  157,  168,
      454,  168,  160,   81,  160,  453,  162,  153,  162,  170,
      170,  157,  152,  166,  160,  166,  452,  169,  162,  169,
      171,  178,  171,  178,  188,  451,  188,  153,  166,  169,
      450,  176,  171,  176,  160,  176,  178,  449,  162,  179,
      180,  179,  180,  181,  180,  181,  269,  269,  176,  169,
      192,  179,  171,  192,  179,  181,  193,  448,  181,  193,
      195,  196,  195,  195,  196,  197,  195,  198,  197,  199,

      198,  179,  199,  200,  192,  181,  200,  201,  196,  447,
      201,  202,  193,  199,  202,  198,  203,  204,  205,  203,
      204,  205,  301,  301,  206,  197,  201,  206,  207,  200,
      446,  207,  203,  202,  208,  209,  210,  208,  209,  210,
      416,  202,  211,  204,  401,  211,  212,  205,  400,  212,
      213,  206,  399,  213,  214,  207,  215,  214,  216,  215,
      398,  216,  217,  209,  397,  217,  218,  211,  219,  218,
      220,  219,  224,  220,  224,  396,  214,  212,  215,  230,
      213,  230,  235,  395,  235,  228,  217,  394,  220,  216,
      219,  228,  228,  238,  239,  238,  228,  239,  393,  240,

      241,  218,  240,  241,  242,  392,  243,  242,  243,  243,
      244,  245,  243,  244,  245,  246,  247,  248,  246,  247,
      248,  249,  250,  240,  249,  250,  241,  244,  252,  251,
      247,  252,  251,  246,  253,  254,  242,  253,  254,  255,
      249,  245,  255,  248,  250,  251,  256,  257,  258,  256,
      257,  258,  250,  259,  252,  260,  259,  261,  260,  391,
      261,  262,  254,  253,  262,  264,  255,  263,  264,  265,
      263,  266,  265,  267,  266,  257,  267,  390,  259,  268,
      311,  311,  268,  262,  320,  320,  260,  261,  276,  263,
      276,  389,  307,  265,  307,  267,  264,  268,  309,  315,

      309,  315,  318,  329,  318,  329,  266,  307,  323,  326,
      323,  326,  323,  326,  315,  388,  332,  333,  329,  332,
      333,  334,  335,  386,  334,  335,  326,  336,  337,  338,
      336,  337,  338,  339,  341,  340,  339,  341,  385,  334,
      335,  340,  337,  342,  340,  336,  342,  333,  343,  344,
      384,  343,  344,  341,  345,  346,  383,  345,  346,  338,
      382,  342,  347,  344,  339,  347,  343,  381,  348,  349,
      346,  348,  349,  351,  350,  352,  351,  350,  352,  345,
      328,  354,  353,  348,  354,  353,  327,  355,  358,  347,
      355,  358,  359,  363,  360,  359,  363,  360,  325,  349,

      350,  353,  322,  351,  317,  354,  361,  352,  316,  361,
      359,  355,  360,  362,  364,  314,  362,  364,  358,  365,
      403,  403,  308,  363,  361,  365,  306,  362,  365,  366,
      305,  367,  366,  368,  367,  300,  368,  370,  369,  371,
      370,  369,  371,  372,  373,  364,  372,  373,  366,  367,
      374,  368,  369,  374,  371,  375,  299,  376,  375,  373,
      376,  377,  370,  378,  377,  379,  378,  298,  379,  297,
      372,  380,  405,  405,  380,  408,  409,  296,  408,  409,
      374,  375,  378,  410,  295,  411,  410,  376,  411,  379,
      294,  293,  292,  377,  409,  380,  412,  413,  414,  412,

      413,  414,  415,  417,  419,  415,  417,  419,  418,  420,
      421,  418,  420,  421,  422,  290,  423,  422,  419,  423,
      413,  288,  287,  424,  415,  412,  424,  421,  414,  286,
      425,  417,  418,  425,  423,  285,  426,  284,  420,  426,
      427,  428,  282,  427,  428,  422,  429,  430,  431,  429,
      430,  431,  424,  432,  426,  433,  432,  427,  433,  428,
      434,  435,  436,  434,  435,  436,  437,  438,  281,  437,
      438,  439,  440,  280,  439,  440,  432,  431,  279,  278,
      437,  441,  434,  277,  441,  433,  436,  442,  439,  435,
      442,  443,  444,  275,  443,  444,  438,  445,  461,  441,

      445,  461,  462,  440,  463,  274,  464,  463,  462,  464,
      444,  462,  465,  271,  445,  465,  442,  466,  467,  468,
      466,  467,  468,  471,  469,  472,  471,  469,  236,  461,
      463,  472,  473,  468,  472,  473,  465,  464,  232,  474,
      475,  466,  474,  475,  476,  477,  231,  476,  477,  467,
      469,  229,  478,  479,  471,  478,  479,  489,  473,  490,
      489,  227,  490,  226,  475,  187,  478,  186,  476,  491,
      474,  492,  491,  185,  492,  493,  477,  496,  493,  479,
      496,  498,  499,  183,  498,  499,  182,  177,  175,  489,
      174,  172,  167,  165,  490,  496,  164,  158,  156,  155,

      499,  146,  145,  492,  143,  140,  139,  138,  136,  133,
      493,  502,  502,  502,  502,  502,  502,  502,  502,  502,
      502,  503,  503,  503,  503,  503,  503,  503,  503,  503,
      503,  504,  504,  504,  504,  504,  504,  504,  504,  504,
      504,  505,  505,  505,  505,  505,  505,  505,  505,  505,
      505,  506,  506,  506,  506,  506,  506,  506,  506,  506,
      506,  507,  507,  131,  507,  507,  507,  507,  507,  507,
      507,  508,  508,  508,  508,  509,  509,  510,  128,  126,
      510,  510,  122,  510,  510,  510,  510,  511,  511,   88,
      511,  511,  511,  511,  511,  511,  511,  512,   76,   74,

      512,  512,   73,  512,  512,  512,  512,  513,  513,  514,
      514,  515,  515,  515,  515,  516,  516,   37,  516,  516,
      516,  516,  516,  516,  516,  517,  517,  518,  518,  518,
       35,  518,  518,  518,  518,  518,  519,  519,   33,  519,
       31,  519,  519,  519,  519,  520,  520,    0,  520,  520,
      520,  520,  520,  520,  520,  521,  521,  521,  521,  521,
      521,  521,  521,  521,  521,  522,  522,    0,  522,    0,
      522,  522,  522,  522,  523,  523,    0,  523,  523,  523,
      523,  523,  523,  523,  524,  524,  524,  524,  524,  524,
      524,  524,  524,  524,  525,  525,  525,  525,    0,  525,

      525,  525,  525,  525,  526,  526,  526,  526,  526,  526,
      526,  526,  526,  526,  528,  528,  528,  528,  530,  530,
      530,  530,  501,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501,  501,  501,  501,  501,
      501,  501,  501,  501,  501,  501
    } ;

extern int pycodeYY_flex_debug;
int pycodeYY_flex_debug = 0;

static yy_state_type *yy_state_buf=0, *yy_state_ptr=0;
static char *yy_full_match;
static int yy_lp;
static int yy_looking_for_trail_begin = 0;
static int yy_full_lp;
static int *yy_full_state;
#define YY_TRAILING_MASK 0x2000
#define YY_TRAILING_HEAD_MASK 0x4000
#define REJECT \
{ \
*yy_cp = (yy_hold_char); /* undo effects of setting up pycodeYYtext */ \
yy_cp = (yy_full_match); /* restore poss. backed-over text */ \
(yy_lp) = (yy_full_lp); /* restore orig. accepting pos. */ \
(yy_state_ptr) = (yy_full_state); /* restore orig. state */ \
yy_current_state = *(yy_state_ptr); /* restore curr. state */ \
++(yy_lp); \
goto find_rule; \
}

#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *pycodeYYtext;
#line 1 "pycode.l"
/******************************************************************************
 *
 * $Id: pycode.h,v 1.9 2001/03/19 19:27:39 root Exp $
 *
 * Copyright (C) 1997-2011 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */
/*  This code is based on the work done by the MoxyPyDoxy team
 *  (Linda Leong, Mike Rivera, Kim Truong, and Gabriel Estrada)
 *  in Spring 2005 as part of CS 179E: Compiler Design Project
 *  at the University of California, Riverside; the course was
 *  taught by Peter H. Froehlich <phf@acm.org>.
 */
#line 26 "pycode.l"

#include <stdio.h>
#include <qvaluestack.h>

#include "pycode.h"
#include "message.h"

#include "scanner.h"
#include "entry.h"
#include "doxygen.h"
#include "outputlist.h"
#include "util.h"
#include "membername.h"
#include "searchindex.h"

#define YY_NEVER_INTERACTIVE 1

static ClassSDict    g_codeClassSDict(17);
static QCString      g_curClassName;
static QStrList      g_curClassBases;


static CodeOutputInterface * g_code;
static const char *  g_inputString;     //!< the code fragment as text
static int	     g_inputPosition;   //!< read offset during parsing 
static const char *  g_currentFontClass;
static bool          g_needsTermination;
static int           g_inputLines;      //!< number of line in the code fragment
static int	     g_yyLineNr;        //!< current line number
static FileDef *     g_sourceFileDef;
static Definition *  g_currentDefinition;
static MemberDef *   g_currentMemberDef;
static bool          g_includeCodeFragment;
static QCString      g_realScope;
//static bool          g_insideBody;
static int           g_bodyCurlyCount;
static bool          g_searchingForBody;
static QCString      g_classScope;
static int           g_paramParens;
//static int           g_anchorCount;

static bool          g_exampleBlock;
static QCString      g_exampleName;
static QCString      g_exampleFile;

static QCString      g_type;
static QCString      g_name;

static bool          g_doubleStringIsDoc;
static bool          g_doubleQuote;
static bool          g_noSuiteFound;
static int           g_stringContext;

static QValueStack<uint> g_indents;  //!< Tracks indentation levels for scoping in python

static void endFontClass();
static void adjustScopesAndSuites(unsigned indentLength);


/*! Represents a stack of variable to class mappings as found in the
 *  code. Each scope is enclosed in pushScope() and popScope() calls.
 *  Variables are added by calling addVariables() and one can search
 *  for variable using findVariable().
 */
class PyVariableContext 
{
  public:
    static const ClassDef *dummyContext;    
    class Scope : public SDict<ClassDef> 
    {
      public:
	Scope() : SDict<ClassDef>(17) {}
    };
    
    PyVariableContext() 
    {
      m_scopes.setAutoDelete(TRUE);
    }

    virtual ~PyVariableContext() 
    {
    }
    
    void pushScope() 
    {
      m_scopes.append(new Scope);
    }

    void popScope() 
    {
      if (m_scopes.count()>0) 
      {
	m_scopes.remove(m_scopes.count()-1);
      }
    }

    void clear() 
    {
      m_scopes.clear();
      m_globalScope.clear();
    }

    void clearExceptGlobal() 
    {
      m_scopes.clear();
    }

    void addVariable(const QCString &type,const QCString &name);
    ClassDef *findVariable(const QCString &name);
    
  private:
    Scope        m_globalScope;
    QList<Scope> m_scopes;
};

void PyVariableContext::addVariable(const QCString &type,const QCString &name)
{
  //printf("PyVariableContext::addVariable(%s,%s)\n",type.data(),name.data());
  QCString ltype = type.simplifyWhiteSpace();
  QCString lname = name.simplifyWhiteSpace();

  Scope *scope = m_scopes.count()==0 ? &m_globalScope : m_scopes.getLast();
  ClassDef *varType;
  if (
      (varType=g_codeClassSDict[ltype]) ||  // look for class definitions inside the code block
      (varType=getResolvedClass(g_currentDefinition,g_sourceFileDef,ltype)) // look for global class definitions
     ) 
  {
    scope->append(lname,varType); // add it to a list
  }
  else 
  {
    if (m_scopes.count()>0) // for local variables add a dummy entry so the name 
                            // is hidden to avoid FALSE links to global variables with the same name
                            // TODO: make this work for namespaces as well!
    {
      scope->append(lname,dummyContext);
    }
  }
}

ClassDef *PyVariableContext::findVariable(const QCString &name)
{
  if (name.isEmpty()) return 0;
  ClassDef *result = 0;
  QListIterator<Scope> sli(m_scopes);
  Scope *scope;
  // search from inner to outer scope
  for (sli.toLast();(scope=sli.current());--sli)
  {
    result = scope->find(name);
    if (result) 
    {
      return result;
    }
  }
  // nothing found -> also try the global scope
  result=m_globalScope.find(name);
  return result;
}

static PyVariableContext g_theVarContext;
const ClassDef *PyVariableContext::dummyContext = (ClassDef*)0x8;

class PyCallContext
{
  public:
    struct Ctx
    {
      Ctx() : name(g_name), type(g_type), cd(0) {}
      QCString name;
      QCString type;
      ClassDef *cd;
    };

    PyCallContext() 
    {
      m_classList.append(new Ctx);
      m_classList.setAutoDelete(TRUE);
    }

    virtual ~PyCallContext() {}

    void setClass(ClassDef *cd)
    {
      Ctx *ctx = m_classList.getLast();
      if (ctx) 
      {
        ctx->cd=cd;
      }
    }
    void pushScope()
    {
      m_classList.append(new Ctx);
    }

    void popScope()
    {
      if (m_classList.count()>1)
      {
	Ctx *ctx = m_classList.getLast();
	if (ctx)
	{
	  g_name = ctx->name;
	  g_type = ctx->type;
	}
	m_classList.removeLast();
      }
      else
      {
      }
    }

    void clear()
    {
      m_classList.clear();
      m_classList.append(new Ctx);
    }

    ClassDef *getClass() const
    {
      Ctx *ctx = m_classList.getLast();

      if (ctx)
        return ctx->cd;
      else
        return 0;
    }

  private:
    QList<Ctx> m_classList;    
};

static PyCallContext g_theCallContext;


/*! counts the number of lines in the input */
static int countLines()
{
  const char *p=g_inputString;
  char c;
  int count=1;
  while ((c=*p)) 
  { 
    p++ ; 
    if (c=='\n') count++;  
  }
  if (p>g_inputString && *(p-1)!='\n') 
  { // last line does not end with a \n, so we add an extra
    // line and explicitly terminate the line after parsing.
    count++, 
    g_needsTermination=TRUE; 
  } 
  return count;
}

static void setCurrentDoc(const QCString &name,const QCString &base,const QCString &anchor="")
{
  if (Doxygen::searchIndex)
  {
    Doxygen::searchIndex->setCurrentDoc(name,base,anchor);
  }
}

static void addToSearchIndex(const char *text)
{
  if (Doxygen::searchIndex)
  {
    Doxygen::searchIndex->addWord(text,FALSE);
  }
}


static ClassDef *stripClassName(const char *s)
{
  int pos=0;
  QCString type = s;
  QCString className;
  QCString templSpec;
  while (extractClassNameFromType(type,pos,className,templSpec)!=-1)
  {
    QCString clName=className+templSpec;

    ClassDef *cd=0;
    if (!g_classScope.isEmpty())
    {
      cd=getResolvedClass(g_currentDefinition,g_sourceFileDef,g_classScope+"::"+clName);
    }
    if (cd==0)
    {
      cd=getResolvedClass(g_currentDefinition,g_sourceFileDef,clName);
    }
    if (cd)
    {
      return cd;
    }
  }

  return 0;
}



/*! start a new line of code, inserting a line number if g_sourceFileDef
 * is TRUE. If a definition starts at the current line, then the line
 * number is linked to the documentation of that definition.
 */
static void startCodeLine()
{
  //if (g_currentFontClass) { g_code->endFontClass(); }
  if (g_sourceFileDef)
  {
    //QCString lineNumber,lineAnchor;
    //lineNumber.sprintf("%05d",g_yyLineNr);
    //lineAnchor.sprintf("l%05d",g_yyLineNr);
   
    Definition *d   = g_sourceFileDef->getSourceDefinition(g_yyLineNr);
    //printf("startCodeLine %d d=%p\n",g_yyLineNr,d);
    //g_code->startLineNumber();
    if (!g_includeCodeFragment && d && d->isLinkableInProject())
    {
      g_currentDefinition = d;
      g_currentMemberDef = g_sourceFileDef->getSourceMember(g_yyLineNr);
      //g_insideBody = FALSE;
      g_searchingForBody = TRUE;
      g_realScope = d->name().copy();
      g_classScope = d->name().copy();
      //printf("Real scope: `%s'\n",g_realScope.data());
      g_bodyCurlyCount = 0;
      QCString lineAnchor;
      lineAnchor.sprintf("l%05d",g_yyLineNr);
      if (g_currentMemberDef)
      {
        g_code->writeLineNumber(g_currentMemberDef->getReference(),
	                        g_currentMemberDef->getOutputFileBase(),
	                        g_currentMemberDef->anchor(),g_yyLineNr);
        setCurrentDoc(
                                g_currentMemberDef->qualifiedName(),
	                        g_sourceFileDef->getSourceFileBase(),
	                        lineAnchor);
      }
      else
      {
        g_code->writeLineNumber(d->getReference(),
	                        d->getOutputFileBase(),
	                        0,g_yyLineNr);
        setCurrentDoc(
                                d->qualifiedName(),
	                        g_sourceFileDef->getSourceFileBase(),
	                        lineAnchor);
      }
    }
    else
    {
      //g_code->codify(lineNumber);
      g_code->writeLineNumber(0,0,0,g_yyLineNr);
    }
    //g_code->endLineNumber();
  }
  g_code->startCodeLine(); 
  if (g_currentFontClass)
  {
    g_code->startFontClass(g_currentFontClass);
  }
}

static void codify(const char* text) 
{ 
  g_code->codify(text);
}

static void endCodeLine()
{
  endFontClass();
  g_code->endCodeLine();
}

static void nextCodeLine()
{
  const char *fc = g_currentFontClass;
  endCodeLine();
  if (g_yyLineNr<g_inputLines) 
  {
    g_currentFontClass = fc;
    startCodeLine();
  }
}


/*! writes a link to a fragment \a text that may span multiple lines, inserting
 * line numbers for each line. If \a text contains newlines, the link will be 
 * split into multiple links with the same destination, one for each line.
 */
static void writeMultiLineCodeLink(CodeOutputInterface &ol,
                  const char *ref,const char *file,
                  const char *anchor,const char *text,
                  const char *tooltip)
{
  bool done=FALSE;
  char *p=(char *)text;
  while (!done)
  {
    char *sp=p;
    char c;
    while ((c=*p++) && c!='\n') { }
    if (c=='\n')
    {
      g_yyLineNr++;
      *(p-1)='\0';
      //printf("writeCodeLink(%s,%s,%s,%s)\n",ref,file,anchor,sp);
      ol.writeCodeLink(ref,file,anchor,sp,tooltip);
      nextCodeLine();
    }
    else
    {
      //printf("writeCodeLink(%s,%s,%s,%s)\n",ref,file,anchor,sp);
      ol.writeCodeLink(ref,file,anchor,sp,tooltip);
      done=TRUE;
    }
  }
}


static void codifyLines(char *text)
{
  //printf("codifyLines(%d,\"%s\")\n",g_yyLineNr,text);
  char *p=text,*sp=p;
  char c;
  bool done=FALSE;
  while (!done)
  {
    sp=p;
    while ((c=*p++) && c!='\n') { }
    if (c=='\n')
    {
      g_yyLineNr++;
      *(p-1)='\0';
      g_code->codify(sp);
      nextCodeLine();
    }
    else
    {
      g_code->codify(sp);
      done=TRUE;
    }
  }
}

static void addDocCrossReference(MemberDef *src,MemberDef *dst)
{
  static bool referencedByRelation = Config_getBool("REFERENCED_BY_RELATION");
  static bool callerGraph =  Config_getBool("CALLER_GRAPH");
  static bool referencesRelation = Config_getBool("REFERENCES_RELATION");
  static bool callGraph = Config_getBool("CALL_GRAPH");
  if (dst->isTypedef() || dst->isEnumerate()) return; // don't add types
  //printf("addDocCrossReference src=%s,dst=%s\n",src->name().data(),dst->name().data());
  if ((referencedByRelation || callerGraph) && (src->isFunction() || src->isSlot()))
  {
    dst->addSourceReferencedBy(src);
  }
  if ((referencesRelation || callGraph) && (src->isFunction() || src->isSlot()))
  {
    src->addSourceReferences(dst);
  }
}



static bool getLinkInScope(const QCString &c,  // scope
                           const QCString &m,  // member
			   const char *memberText, // exact text
			   CodeOutputInterface &ol,
			   const char *text
			  )
{
  MemberDef    *md;
  ClassDef     *cd;
  FileDef      *fd;
  NamespaceDef *nd;
  GroupDef     *gd;
  //printf("Trying `%s'::`%s'\n",c.data(),m.data());
  if (getDefs(c,m,"()",md,cd,fd,nd,gd,FALSE,g_sourceFileDef) && 
      md->isLinkable())
  {
    //Definition *d=0;
    //if (cd) d=cd; else if (nd) d=nd; else if (fd) d=fd; else d=gd;

    Definition *d = md->getOuterScope()==Doxygen::globalScope ?
	            md->getBodyDef() : md->getOuterScope();
    //printf("Found! d=%s\n",d?d->name().data():"<none>");
    if (md->getGroupDef()) d = md->getGroupDef();
    if (d && d->isLinkable())
    {
      g_theCallContext.setClass(stripClassName(md->typeString()));
      //printf("g_currentDefinition=%p g_currentMemberDef=%p\n",
      //        g_currentDefinition,g_currentMemberDef);

      if (g_currentDefinition && g_currentMemberDef &&
	  md!=g_currentMemberDef)
      {
	addDocCrossReference(g_currentMemberDef,md);
      }
      //printf("d->getReference()=`%s' d->getOutputBase()=`%s' name=`%s' member name=`%s'\n",d->getReference().data(),d->getOutputFileBase().data(),d->name().data(),md->name().data());
     
      writeMultiLineCodeLink(ol,md->getReference(),
	                        md->getOutputFileBase(),
	                        md->anchor(),
				text ? text : memberText,
                                md->briefDescriptionAsTooltip());
      addToSearchIndex(text ? text : memberText);
      return TRUE;
    } 
  }
  return FALSE;
}

static bool getLink(const char *className,
                    const char *memberName,
		    CodeOutputInterface &ol,
		    const char *text=0)
{
  QCString m=removeRedundantWhiteSpace(memberName);
  QCString c=className;
  if (!getLinkInScope(c,m,memberName,ol,text))
  {
    if (!g_curClassName.isEmpty())
    {
      if (!c.isEmpty()) c.prepend("::");
      c.prepend(g_curClassName);
      return getLinkInScope(c,m,memberName,ol,text);
    }
    return FALSE;
  }
  return TRUE;
}


/*
  For a given string in the source code,
  finds its class or global id and links to it.

  As of June 1, '05, this ONLY finds classes
*/
static void generateClassOrGlobalLink(CodeOutputInterface &ol,char *clName,
                                      bool /*typeOnly*/=FALSE)
{
  QCString className=clName;

  // Don't do anything for empty text
  if (className.isEmpty()) return;

  //printf("generateClassOrGlobalLink(className=%s)\n",className.data());

  ClassDef *cd=0,*lcd=0;  /** Class def that we may find */
  MemberDef *md=0;        /** Member def that we may find */
  bool isLocal=FALSE;

  if ((lcd=g_theVarContext.findVariable(className))==0) // not a local variable
  {
    Definition *d = g_currentDefinition;

    cd = getResolvedClass(d,g_sourceFileDef,className,&md);

    //printf("d=%p g_sourceFileDef=%p\n",d,g_currentDefinition);
    //printf("is found as a type %s\n",cd?cd->name().data():"<null>");

    if (cd==0 && md==0) // also see if it is variable or enum or enum value
    {
      if (getLink(g_classScope,clName,ol,clName))
      {
	return;
      }
    }
  }
  else
  {
    if (lcd!=PyVariableContext::dummyContext) 
    {
      g_theCallContext.setClass(lcd);
    }
    isLocal=TRUE;
    //fprintf(stderr,"is a local variable cd=%p!\n",cd);
  }

  if (cd && cd->isLinkable()) // is it a linkable class
  {
    writeMultiLineCodeLink(ol,cd->getReference(),cd->getOutputFileBase(),cd->anchor(),clName,cd->briefDescriptionAsTooltip());
    addToSearchIndex(className);
    if (md)
    {
      Definition *d = md->getOuterScope()==Doxygen::globalScope ?
                      md->getBodyDef() : md->getOuterScope();
      if (md->getGroupDef()) d = md->getGroupDef();
      if (d && d->isLinkable() && md->isLinkable() && g_currentMemberDef)
      {
        addDocCrossReference(g_currentMemberDef,md);
      }
    }
  }
  else // not a class, maybe a global member
  {
    //printf("Checking if '%s' is a global function\n",clName);

    /*
      This code requires a going-over in order to
      make it work for Python

    //printf("class %s not linkable! cd=%p md=%p typeOnly=%d\n",clName,cd,md,typeOnly);
    if (!isLocal && (md!=0 || (cd==0 && !typeOnly))) // not a class, see if it is a global enum/variable/typedef.
    {
      if (md==0) // not found as a typedef
      {
	md = setCallContextForVar(clName);
	//printf("setCallContextForVar(%s) md=%p g_currentDefinition=%p\n",clName,md,g_currentDefinition);
	if (md && g_currentDefinition)
	{
	  //fprintf(stderr,"%s accessible from %s? %d md->getOuterScope=%s\n",
	  //    md->name().data(),g_currentDefinition->name().data(),
	  //    isAccessibleFrom(g_currentDefinition,g_sourceFileDef,md),
	  //    md->getOuterScope()->name().data());
	}
	     
        if (md && g_currentDefinition && 
	    isAccessibleFrom(g_currentDefinition,g_sourceFileDef,md)==-1)
	{
	  md=0; // variable not accessible
	}
      }
      if (md)
      {
        //printf("is a global md=%p g_currentDefinition=%s\n",md,g_currentDefinition?g_currentDefinition->name().data():"<none>");
	if (md->isLinkable())
	{
	  writeMultiLineCodeLink(ol,md->getReference(),md->getOutputFileBase(),md->anchor(),clName,md->briefDescriptionAsTooltip());
          addToSearchIndex(clName);
	  if (g_currentMemberDef)
	  {
	    addDocCrossReference(g_currentMemberDef,md);
	  }
	  return;
	}
      }
    }

    */
    
    // nothing found, just write out the word
    codifyLines(clName);
    addToSearchIndex(clName);
  }
}

/*
   As of June 1, this function seems to work
   for file members, but scopes are not
   being correctly tracked for classes
   so it doesn't work for classes yet.

*/
static void generateFunctionLink(CodeOutputInterface &ol,char *funcName)
{
  //CodeClassDef *ccd=0;
  ClassDef *ccd=0;
  QCString locScope=g_classScope.copy();
  QCString locFunc=removeRedundantWhiteSpace(funcName);
  //fprintf(stdout,"*** locScope=%s locFunc=%s\n",locScope.data(),locFunc.data());
  int i=locFunc.findRev("::");
  if (i>0)
  {
    locScope=locFunc.left(i);
    locFunc=locFunc.right(locFunc.length()-i-2).stripWhiteSpace();
  }
  //printf("generateFunctionLink(%s) classScope=`%s'\n",locFunc.data(),locScope.data());
  if (!locScope.isEmpty() && (ccd=g_codeClassSDict[locScope]))
  {
    //printf("using classScope %s\n",g_classScope.data());
    if (ccd->baseClasses())
    {
      BaseClassListIterator bcli(*ccd->baseClasses());
      for ( ; bcli.current() ; ++bcli)
      {
	if (getLink(bcli.current()->classDef->name(),locFunc,ol,funcName)) 
	{
	  return;
	}
      }
    }
  }
  if (!getLink(locScope,locFunc,ol,funcName))
  {
    generateClassOrGlobalLink(ol,funcName);
  }
  return;
}

static bool findMemberLink(CodeOutputInterface &ol,Definition *sym,const char *symName)
{
  //printf("sym %s outerScope=%s equal=%d\n",
  //    sym->name().data(),sym->getOuterScope()->name().data(),
  //    sym->getOuterScope()==g_currentDefinition);

  if (sym->getOuterScope() &&
      sym->getOuterScope()->definitionType()==Definition::TypeClass &&
      g_currentDefinition->definitionType()==Definition::TypeClass)
  {
    ClassDef *cd = (ClassDef*)sym->getOuterScope();
    ClassDef *thisCd = (ClassDef *)g_currentDefinition;
    QCString anchor=sym->anchor();
    if (sym->definitionType()==Definition::TypeMember)
    {
      if (g_currentMemberDef)
      {
	addDocCrossReference(g_currentMemberDef,(MemberDef*)sym);
      }
    }

    // TODO: find the nearest base class in case cd is a base class of
    // thisCd 
    if (cd==thisCd) 
    {
      writeMultiLineCodeLink(ol,sym->getReference(),
          sym->getOutputFileBase(),
          anchor,
          symName,
          sym->briefDescriptionAsTooltip());
      return TRUE;
    }
  }
  return FALSE;
}

static void findMemberLink(CodeOutputInterface &ol,char *symName)
{
  //printf("Member reference: %s scope=%s member=%s\n",
  //    pycodeYYtext,
  //    g_currentDefinition?g_currentDefinition->name().data():"<none>",
  //    g_currentMemberDef?g_currentMemberDef->name().data():"<none>"
  //    );
  if (g_currentDefinition)
  {
    DefinitionIntf *di = Doxygen::symbolMap->find(symName);
    if (di)
    {
      if (di->definitionType()==DefinitionIntf::TypeSymbolList) // multiple symbols
      {
	DefinitionListIterator dli(*(DefinitionList*)di);
	Definition *sym;
	for (dli.toFirst();(sym=dli.current());++dli)
	{
	  if (findMemberLink(ol,sym,symName)) return;
	}
      }
      else // single symbol
      {
	if (findMemberLink(ol,(Definition*)di,symName)) return;
      }
    }
  }
  //printf("sym %s not found\n",&pycodeYYtext[5]);
  codify(symName);
}

static void startFontClass(const char *s)
{
  endFontClass();
  g_code->startFontClass(s);
  g_currentFontClass=s;
}

static void endFontClass()
{
  if (g_currentFontClass)
  {
    g_code->endFontClass();
    g_currentFontClass=0;
  }
}

#undef YY_INPUT
#define YY_INPUT(buf,result,max_size) result=yyread(buf,max_size);

static int yyread(char *buf,int max_size)
{
  int c=0;
  while( c < max_size && g_inputString[g_inputPosition] )
  {
    *buf = g_inputString[g_inputPosition++] ;
    c++; buf++;
  }
  return c;
}















#line 1957 "<stdout>"

#define INITIAL 0
#define Body 1
#define FunctionDec 2
#define FunctionParams 3
#define ClassDec 4
#define ClassInheritance 5
#define Suite 6
#define SuiteCaptureIndent 7
#define SuiteStart 8
#define SuiteMaintain 9
#define SuiteContinuing 10
#define LongString 11
#define SingleQuoteString 12
#define DoubleQuoteString 13
#define TripleString 14

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int pycodeYYlex_destroy (void );

int pycodeYYget_debug (void );

void pycodeYYset_debug (int debug_flag  );

YY_EXTRA_TYPE pycodeYYget_extra (void );

void pycodeYYset_extra (YY_EXTRA_TYPE user_defined  );

FILE *pycodeYYget_in (void );

void pycodeYYset_in  (FILE * in_str  );

FILE *pycodeYYget_out (void );

void pycodeYYset_out  (FILE * out_str  );

yy_size_t pycodeYYget_leng (void );

char *pycodeYYget_text (void );

int pycodeYYget_lineno (void );

void pycodeYYset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int pycodeYYwrap (void );
#else
extern int pycodeYYwrap (void );
#endif
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 262144
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO fwrite( pycodeYYtext, pycodeYYleng, 1, pycodeYYout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		yy_size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( pycodeYYin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( pycodeYYin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, pycodeYYin))==0 && ferror(pycodeYYin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(pycodeYYin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int pycodeYYlex (void);

#define YY_DECL int pycodeYYlex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after pycodeYYtext and pycodeYYleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
#line 952 "pycode.l"


#line 2154 "<stdout>"

	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

        /* Create the reject buffer large enough to save one state per allowed character. */
        if ( ! (yy_state_buf) )
            (yy_state_buf) = (yy_state_type *)pycodeYYalloc(YY_STATE_BUF_SIZE  );
            if ( ! (yy_state_buf) )
                YY_FATAL_ERROR( "out of dynamic memory in pycodeYYlex()" );

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! pycodeYYin )
			pycodeYYin = stdin;

		if ( ! pycodeYYout )
			pycodeYYout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			pycodeYYensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				pycodeYY_create_buffer(pycodeYYin,YY_BUF_SIZE );
		}

		pycodeYY_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of pycodeYYtext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);

		(yy_state_ptr) = (yy_state_buf);
		*(yy_state_ptr)++ = yy_current_state;

yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 502 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			*(yy_state_ptr)++ = yy_current_state;
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 1623 );

yy_find_action:
		yy_current_state = *--(yy_state_ptr);
		(yy_lp) = yy_accept[yy_current_state];
goto find_rule; /* Shut up GCC warning -Wall */
find_rule: /* we branch to this label when backing up */
		for ( ; ; ) /* until we find what rule we matched */
			{
			if ( (yy_lp) && (yy_lp) < yy_accept[yy_current_state + 1] )
				{
				yy_act = yy_acclist[(yy_lp)];
				if ( yy_act & YY_TRAILING_HEAD_MASK ||
				     (yy_looking_for_trail_begin) )
					{
					if ( yy_act == (yy_looking_for_trail_begin) )
						{
						(yy_looking_for_trail_begin) = 0;
						yy_act &= ~YY_TRAILING_HEAD_MASK;
						break;
						}
					}
				else if ( yy_act & YY_TRAILING_MASK )
					{
					(yy_looking_for_trail_begin) = yy_act & ~YY_TRAILING_MASK;
					(yy_looking_for_trail_begin) |= YY_TRAILING_HEAD_MASK;
					(yy_full_match) = yy_cp;
					(yy_full_state) = (yy_state_ptr);
					(yy_full_lp) = (yy_lp);
					}
				else
					{
					(yy_full_match) = yy_cp;
					(yy_full_state) = (yy_state_ptr);
					(yy_full_lp) = (yy_lp);
					break;
					}
				++(yy_lp);
				goto find_rule;
				}
			--yy_cp;
			yy_current_state = *--(yy_state_ptr);
			(yy_lp) = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */

case 1:
YY_RULE_SETUP
#line 955 "pycode.l"
{
				        startFontClass("keyword");
					codify(pycodeYYtext);
					endFontClass();
					BEGIN( FunctionDec );
		                    }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 962 "pycode.l"
{
				        startFontClass("keyword");
					codify(pycodeYYtext);
					endFontClass();
					BEGIN( ClassDec );
		                    }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 968 "pycode.l"
{
				        startFontClass("keywordtype");
					codify(pycodeYYtext);
					endFontClass();
				    }
	YY_BREAK
case 4:
*yy_cp = (yy_hold_char); /* undo effects of setting up pycodeYYtext */
(yy_c_buf_p) = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up pycodeYYtext again */
YY_RULE_SETUP
#line 973 "pycode.l"
{
				        codify("self.");
				        findMemberLink(*g_code,&pycodeYYtext[5]);
	                            }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 977 "pycode.l"
{
				        codify("self.");
				        findMemberLink(*g_code,&pycodeYYtext[5]);
	                            }
	YY_BREAK

case 6:
YY_RULE_SETUP
#line 983 "pycode.l"
{

					generateClassOrGlobalLink(*g_code,pycodeYYtext);
					// codify(pycodeYYtext);
					g_curClassName = pycodeYYtext;
					g_curClassBases.clear();
					BEGIN( ClassInheritance );
		                    }
	YY_BREAK

case 7:
YY_RULE_SETUP
#line 993 "pycode.l"
{
					codify(pycodeYYtext);
                                    }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 997 "pycode.l"
{
                                        // The parser
					// is assuming
					// that ALL identifiers
					// in this state
					// are base classes;
					// it doesn't check to see
					// that the first parenthesis
					// has been seen.

					// This is bad - it should
					// probably be more strict
					// about what to accept.

                                        g_curClassBases.inSort(pycodeYYtext);
					generateClassOrGlobalLink(*g_code,pycodeYYtext);
					// codify(pycodeYYtext);
                                    }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 1016 "pycode.l"
{
				      codify(pycodeYYtext);

				      // Assume this will
				      // be a one-line suite;
				      // found counter-example
				      // in SuiteStart.

                                      // Push a class scope

                                      ClassDef *classDefToAdd = new ClassDef("<code>",1,g_curClassName,ClassDef::Class,0,0,FALSE);
                                      g_codeClassSDict.append(g_curClassName,classDefToAdd);
                                      char *s=g_curClassBases.first();
                                      while (s) 
				      {
                                        ClassDef *baseDefToAdd;
                                        baseDefToAdd=g_codeClassSDict[s];

					// Try to find class in global
					// scope
					if (baseDefToAdd==0) 
					{
					  baseDefToAdd=getResolvedClass(g_currentDefinition,g_sourceFileDef,s);
					}

					if (baseDefToAdd && baseDefToAdd!=classDefToAdd) 
					{
				          classDefToAdd->insertBaseClass(baseDefToAdd,s,Public,Normal);
					}

                                        s=g_curClassBases.next();
			              }

				      // Reset class-parsing variables.
				      g_curClassName.resize(0);
				      g_curClassBases.clear();
				      
				      g_noSuiteFound = TRUE;
				      BEGIN( SuiteStart );
                                    }
	YY_BREAK


case 10:
YY_RULE_SETUP
#line 1060 "pycode.l"
{
                                        generateFunctionLink(*g_code,pycodeYYtext);
                                    }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 1064 "pycode.l"
{
				        codify(pycodeYYtext);
					BEGIN( FunctionParams );
		                    }
	YY_BREAK


case 12:
YY_RULE_SETUP
#line 1071 "pycode.l"
{
                                         // Parses delimiters
				         codify(pycodeYYtext);
                                    }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 1076 "pycode.l"
{
				         codify(pycodeYYtext);
                                    }
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 1080 "pycode.l"
{
                                         codify(pycodeYYtext);
                                    }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 1084 "pycode.l"
{
				      codify(pycodeYYtext);

				      // Assume this will
				      // be a one-line suite;
				      // found counter-example
				      // in SuiteStart.
				      g_noSuiteFound = TRUE;
				      BEGIN( SuiteStart );
                                    }
	YY_BREAK


case 16:
YY_RULE_SETUP
#line 1098 "pycode.l"
{
                                 // Position-sensitive rules!
                                 // Must come AFTER keyword-triggered rules
                                 // Must come BEFORE identifier NONEMPTY-like rules
                                 //   to syntax highlight.

  		                 startFontClass("keyword");
  		                 codify(pycodeYYtext);
		                 endFontClass();
                               }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 1109 "pycode.l"
{
  		                 startFontClass("keywordflow");
  		                 codify(pycodeYYtext);
		                 endFontClass();
                               }
	YY_BREAK
case 18:
*yy_cp = (yy_hold_char); /* undo effects of setting up pycodeYYtext */
(yy_c_buf_p) = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up pycodeYYtext again */
YY_RULE_SETUP
#line 1114 "pycode.l"
{
				 generateClassOrGlobalLink(*g_code,pycodeYYtext);
      			       }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 1117 "pycode.l"
{
  		                 codify(pycodeYYtext);
				 //generateClassOrGlobalLink(*g_code,pycodeYYtext);
			       }
	YY_BREAK


case 20:
YY_RULE_SETUP
#line 1127 "pycode.l"
{
                                         codify(pycodeYYtext);
                                       }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 1130 "pycode.l"
{
                                          startFontClass("keyword");
					  codifyLines(pycodeYYtext);
		                          endFontClass();

					  // No indentation necesary
					  g_noSuiteFound = FALSE;
                                       }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 1139 "pycode.l"
{
                                          startFontClass("keywordflow");
					  codifyLines(pycodeYYtext);
		                          endFontClass();

					  // No indentation necesary
					  g_noSuiteFound = FALSE;
                                       }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 1147 "pycode.l"
{
                                         codify(pycodeYYtext);
				       } 
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 1152 "pycode.l"
{
                                          // This eats EVERYTHING
					  // except the newline
                                          startFontClass("comment");
					  codifyLines(pycodeYYtext);
		                          endFontClass();
                                       }
	YY_BREAK
case 25:
/* rule 25 can match eol */
YY_RULE_SETUP
#line 1160 "pycode.l"
{
					  codifyLines(pycodeYYtext);
					  if ( g_noSuiteFound ) 
					  {
                                            // printf("New suite to capture! [%d]\n", g_yyLineNr);
					    BEGIN ( SuiteCaptureIndent );
					  }
                                       }
	YY_BREAK


case 26:
/* rule 26 can match eol */
YY_RULE_SETUP
#line 1171 "pycode.l"
{
                                 // Blankline - ignore, keep looking for indentation.
				 codifyLines(pycodeYYtext);
                               }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 1176 "pycode.l"
{
                                 // This state lasts momentarily,
                                 // to check the indentation
                                 // level that is about to be
                                 // used.
				 codifyLines(pycodeYYtext);
				 g_indents.push(pycodeYYleng);
                                 // printf("Captured indent of %d [line %d]\n", pycodeYYleng, g_yyLineNr);
				 BEGIN( Suite );
                               }
	YY_BREAK


case 28:
YY_RULE_SETUP
#line 1190 "pycode.l"
{
                                 // This implements poor
				 // indendation-tracking;
                                 // should be improved.
				 // (translate tabs to space, etc)
  		                 codifyLines(pycodeYYtext);
                                 adjustScopesAndSuites(pycodeYYleng);
                               }
	YY_BREAK
case 29:
/* rule 29 can match eol */
YY_RULE_SETUP
#line 1199 "pycode.l"
{
                                 // If this ever succeeds,
				 // it means that this is
				 // a blank line, and
				 // can be ignored.
  		                 codifyLines(pycodeYYtext);
                               }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 1207 "pycode.l"
{
                                 // Default rule; matches
				 // the empty string, assuming
				 // real text starts here.
				 // Just go straight to Body.
                                 adjustScopesAndSuites(0);
                               }
	YY_BREAK

case 31:
/* rule 31 can match eol */
YY_RULE_SETUP
#line 1217 "pycode.l"
{
                                 codifyLines(pycodeYYtext);
			         BEGIN( SuiteMaintain );
		               }
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 1221 "pycode.l"
{
  			         codify(pycodeYYtext);
                               }
	YY_BREAK
case 33:
/* rule 33 can match eol */
YY_RULE_SETUP
#line 1224 "pycode.l"
{
  		                 codifyLines(pycodeYYtext);
                               }
	YY_BREAK
// Single quoted string like 'That\'s a """nice""" string!'
case 34:
/* rule 34 can match eol */
YY_RULE_SETUP
#line 1229 "pycode.l"
{ // line continuation
  		                 codifyLines(pycodeYYtext);
                               }
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 1232 "pycode.l"
{ // espaced char
  		                 codify(pycodeYYtext);
                               }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 1235 "pycode.l"
{ // tripple double quotes
  		                 codify(pycodeYYtext);
      			       }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 1238 "pycode.l"
{ // end of the string
  		                 codify(pycodeYYtext);
				 endFontClass();
      		                 BEGIN(g_stringContext);
                               }
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 1243 "pycode.l"
{ // normal chars
  		                 codify(pycodeYYtext);
                               }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 1246 "pycode.l"
{ // normal char
  		                 codify(pycodeYYtext);
                               }
	YY_BREAK

// Double quoted string like "That's \"a '''nice'''\" string!"
case 40:
/* rule 40 can match eol */
YY_RULE_SETUP
#line 1252 "pycode.l"
{ // line continuation
  		                 codifyLines(pycodeYYtext);
                               }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 1255 "pycode.l"
{ // espaced char
  		                 codify(pycodeYYtext);
                               }
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 1258 "pycode.l"
{ // tripple single quotes
  		                 codify(pycodeYYtext);
      			       }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 1261 "pycode.l"
{ // end of the string
  		                 codify(pycodeYYtext);
				 endFontClass();
      		                 BEGIN(g_stringContext);
                               }
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 1266 "pycode.l"
{ // normal chars
  		                 codify(pycodeYYtext);
                               }
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 1269 "pycode.l"
{ // normal char
  		                 codify(pycodeYYtext);
                               }
	YY_BREAK


case 46:
#line 1276 "pycode.l"
case 47:
YY_RULE_SETUP
#line 1276 "pycode.l"
{
  		          codify(pycodeYYtext);
			  if (g_doubleQuote==(pycodeYYtext[0]=='"')) 
			  {
			    endFontClass();
			    BEGIN(g_stringContext);
			  }
		       }
	YY_BREAK
case 48:
/* rule 48 can match eol */
YY_RULE_SETUP
#line 1284 "pycode.l"
{
			 codifyLines(pycodeYYtext);
		       }
	YY_BREAK
case 49:
/* rule 49 can match eol */
YY_RULE_SETUP
#line 1287 "pycode.l"
{
			 codifyLines(pycodeYYtext);
                       }
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 1290 "pycode.l"
{
                         codify(pycodeYYtext);
                       }
	YY_BREAK

/*
<*>({NONEMPTY}|{EXPCHAR}|{BB})           { // This should go one character at a time.
  		                 codify(pycodeYYtext);
				 // printf("[pycode] '%s' [ state %d ]  [line %d] no match\n",
				 //       pycodeYYtext, YY_START, g_yyLineNr);

				 //endFontClass();
				 BEGIN(Body);					
                               }
   */
case 51:
#line 1307 "pycode.l"
case 52:
YY_RULE_SETUP
#line 1307 "pycode.l"
{
  				 startFontClass("stringliteral");
				 g_stringContext=YY_START;
				 g_doubleQuote=pycodeYYtext[pycodeYYleng-1]=='"';
  		                 codify(pycodeYYtext);
				 BEGIN(TripleString);
                               }
	YY_BREAK
case 53:
YY_RULE_SETUP
#line 1314 "pycode.l"
{ // single quoted string
  				 startFontClass("stringliteral");
				 g_stringContext=YY_START;
  		                 codify(pycodeYYtext);
				 BEGIN(SingleQuoteString);
  			       }
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 1320 "pycode.l"
{ // double quoted string
  				 startFontClass("stringliteral");
				 g_stringContext=YY_START;
  		                 codify(pycodeYYtext);
				 BEGIN(DoubleQuoteString);
                               }
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 1326 "pycode.l"
{
  				 if (YY_START==SingleQuoteString || 
				     YY_START==DoubleQuoteString || 
				     YY_START==TripleString
				    )
				 {
				   REJECT;
				 }
                                 // This eats EVERYTHING
				 // except the newline
                                 startFontClass("comment");
				 codifyLines(pycodeYYtext);
		                 endFontClass();
                               }
	YY_BREAK
case 56:
/* rule 56 can match eol */
YY_RULE_SETUP
#line 1340 "pycode.l"
{
  		                 codifyLines(pycodeYYtext);
				 //printf("[pycode] %d NEWLINE [line %d] no match\n",
				 //       YY_START, g_yyLineNr);

				 //endFontClass();
				 BEGIN(Body);
                               }
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 1349 "pycode.l"
{
  		                 codify(pycodeYYtext);
				 BEGIN(Body);					
  			       }
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 1353 "pycode.l"
{
  		                 codify(pycodeYYtext);
				 // printf("[pycode] '%s' [ state %d ]  [line %d] no match\n",
				 //        pycodeYYtext, YY_START, g_yyLineNr);

				 //endFontClass();
				 BEGIN(Body);					
                               }
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 1362 "pycode.l"
ECHO;
	YY_BREAK
#line 2887 "<stdout>"
			case YY_STATE_EOF(INITIAL):
			case YY_STATE_EOF(Body):
			case YY_STATE_EOF(FunctionDec):
			case YY_STATE_EOF(FunctionParams):
			case YY_STATE_EOF(ClassDec):
			case YY_STATE_EOF(ClassInheritance):
			case YY_STATE_EOF(Suite):
			case YY_STATE_EOF(SuiteCaptureIndent):
			case YY_STATE_EOF(SuiteStart):
			case YY_STATE_EOF(SuiteMaintain):
			case YY_STATE_EOF(SuiteContinuing):
			case YY_STATE_EOF(LongString):
			case YY_STATE_EOF(SingleQuoteString):
			case YY_STATE_EOF(DoubleQuoteString):
			case YY_STATE_EOF(TripleString):
				yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed pycodeYYin at a new source and called
			 * pycodeYYlex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = pycodeYYin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( pycodeYYwrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * pycodeYYtext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of pycodeYYlex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			yy_size_t num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			pycodeYYrestart(pycodeYYin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) pycodeYYrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);

	(yy_state_ptr) = (yy_state_buf);
	*(yy_state_ptr)++ = yy_current_state;

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 502 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		*(yy_state_ptr)++ = yy_current_state;
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    
	register YY_CHAR yy_c = 1;
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 502 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 501);
	if ( ! yy_is_jam )
		*(yy_state_ptr)++ = yy_current_state;

	return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			yy_size_t offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					pycodeYYrestart(pycodeYYin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( pycodeYYwrap( ) )
						return 0;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve pycodeYYtext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void pycodeYYrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        pycodeYYensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            pycodeYY_create_buffer(pycodeYYin,YY_BUF_SIZE );
	}

	pycodeYY_init_buffer(YY_CURRENT_BUFFER,input_file );
	pycodeYY_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void pycodeYY_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		pycodeYYpop_buffer_state();
	 *		pycodeYYpush_buffer_state(new_buffer);
     */
	pycodeYYensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	pycodeYY_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (pycodeYYwrap()) processing, but the only time this flag
	 * is looked at is after pycodeYYwrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void pycodeYY_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	pycodeYYin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE pycodeYY_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) pycodeYYalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in pycodeYY_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) pycodeYYalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in pycodeYY_create_buffer()" );

	b->yy_is_our_buffer = 1;

	pycodeYY_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with pycodeYY_create_buffer()
 * 
 */
    void pycodeYY_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		pycodeYYfree((void *) b->yy_ch_buf  );

	pycodeYYfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a pycodeYYrestart() or at EOF.
 */
    static void pycodeYY_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	pycodeYY_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then pycodeYY_init_buffer was _probably_
     * called from pycodeYYrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void pycodeYY_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		pycodeYY_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void pycodeYYpush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	pycodeYYensure_buffer_stack();

	/* This block is copied from pycodeYY_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from pycodeYY_switch_to_buffer. */
	pycodeYY_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void pycodeYYpop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	pycodeYY_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		pycodeYY_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void pycodeYYensure_buffer_stack (void)
{
	yy_size_t num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)pycodeYYalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in pycodeYYensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)pycodeYYrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in pycodeYYensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE pycodeYY_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) pycodeYYalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in pycodeYY_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	pycodeYY_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to pycodeYYlex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       pycodeYY_scan_bytes() instead.
 */
YY_BUFFER_STATE pycodeYY_scan_string (yyconst char * yystr )
{
    
	return pycodeYY_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to pycodeYYlex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE pycodeYY_scan_bytes  (yyconst char * yybytes, yy_size_t  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n, i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) pycodeYYalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in pycodeYY_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = pycodeYY_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in pycodeYY_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up pycodeYYtext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		pycodeYYtext[pycodeYYleng] = (yy_hold_char); \
		(yy_c_buf_p) = pycodeYYtext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		pycodeYYleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int pycodeYYget_lineno  (void)
{
        
    return pycodeYYlineno;
}

/** Get the input stream.
 * 
 */
FILE *pycodeYYget_in  (void)
{
        return pycodeYYin;
}

/** Get the output stream.
 * 
 */
FILE *pycodeYYget_out  (void)
{
        return pycodeYYout;
}

/** Get the length of the current token.
 * 
 */
yy_size_t pycodeYYget_leng  (void)
{
        return pycodeYYleng;
}

/** Get the current token.
 * 
 */

char *pycodeYYget_text  (void)
{
        return pycodeYYtext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void pycodeYYset_lineno (int  line_number )
{
    
    pycodeYYlineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see pycodeYY_switch_to_buffer
 */
void pycodeYYset_in (FILE *  in_str )
{
        pycodeYYin = in_str ;
}

void pycodeYYset_out (FILE *  out_str )
{
        pycodeYYout = out_str ;
}

int pycodeYYget_debug  (void)
{
        return pycodeYY_flex_debug;
}

void pycodeYYset_debug (int  bdebug )
{
        pycodeYY_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from pycodeYYlex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

    (yy_state_buf) = 0;
    (yy_state_ptr) = 0;
    (yy_full_match) = 0;
    (yy_lp) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    pycodeYYin = stdin;
    pycodeYYout = stdout;
#else
    pycodeYYin = (FILE *) 0;
    pycodeYYout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * pycodeYYlex_init()
     */
    return 0;
}

/* pycodeYYlex_destroy is for both reentrant and non-reentrant scanners. */
int pycodeYYlex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		pycodeYY_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		pycodeYYpop_buffer_state();
	}

	/* Destroy the stack itself. */
	pycodeYYfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    pycodeYYfree ( (yy_state_buf) );
    (yy_state_buf)  = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * pycodeYYlex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *pycodeYYalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *pycodeYYrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void pycodeYYfree (void * ptr )
{
	free( (char *) ptr );	/* see pycodeYYrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 1362 "pycode.l"



/*@ ----------------------------------------------------------------------------
 */

void resetPythonCodeParserState() 
{
  g_currentDefinition = 0;
  g_currentMemberDef = 0;
  g_doubleStringIsDoc = FALSE;
  g_paramParens = 0;
  g_indents.clear();
  BEGIN( Body );
}

/*!
  Examines current stack of white-space indentations;
  re-syncs the parser with the correct scope.
*/
static void adjustScopesAndSuites(unsigned indentLength) 
{
  // States to pop
  if (!g_indents.isEmpty() && indentLength < g_indents.top()) 
  {
    while (!g_indents.isEmpty() && indentLength < g_indents.top()) 
    {
      // printf("Exited scope indent of [%d]\n", g_indents.top());
      g_indents.pop(); // Pop the old suite's indentation

      g_currentMemberDef=0;
      if (g_currentDefinition) 
	g_currentDefinition=g_currentDefinition->getOuterScope();
    }
  }

  // Are there any remaining indentation levels for suites?
  if (!g_indents.isEmpty()) 
  {
    BEGIN( Suite );
  }
  else 
  {
    BEGIN( Body );
  }
}

void parsePythonCode(CodeOutputInterface &od,const char *className,
                 const QCString &s,bool exBlock, const char *exName,
		 FileDef *fd,int startLine,int endLine,bool inlineFragment,
		 MemberDef *,bool) 
{

  //printf("***parseCode()\n");
  
  //--- some code to eliminate warnings---
  className = "";
  exBlock = FALSE;
  exName = "";
  inlineFragment = "";
  //--------------------------------------
  if (s.isEmpty()) return;
  g_code = &od;
  g_inputString   = s;
  g_inputPosition = 0;
  g_currentFontClass = 0;
  g_needsTermination = FALSE;
  if (endLine!=-1)
    g_inputLines  = endLine+1;
  else
    g_inputLines  = countLines();
  
  if (startLine!=-1)
    g_yyLineNr    = startLine;
  else
    g_yyLineNr    = 1;
  
  g_exampleBlock  = exBlock; 
  g_exampleName   = exName;
  g_sourceFileDef = fd;


  // Starts line 1 on the output  
  startCodeLine();

  pycodeYYrestart( pycodeYYin );

  pycodeYYlex();

  if (!g_indents.isEmpty()) 
  {
    // printf("Exited pysourceparser in inconsistent state!\n");
  }

  if (g_needsTermination)
  {
    endCodeLine();
  }
  return;
}


#if !defined(YY_FLEX_SUBMINOR_VERSION) 
extern "C" { // some bogus code to keep the compiler happy
  void pycodeYYdummy() { yy_flex_realloc(0,0); } 
}
#elif YY_FLEX_SUBMINOR_VERSION<33
#error "You seem to be using a version of flex newer than 2.5.4. These are currently incompatible with 2.5.4, and do NOT work with doxygen! Please use version 2.5.4 or expect things to be parsed wrongly! A bug report has been submitted (#732132)."
#endif


